@@ -2,7 +2,7 @@
 #
 #   Sub::Contract - Programming by contract and memoizing in one
 #
-#   $Id: Contract.pm,v 1.17 2008/05/07 09:08:21 erwan_lemonnier Exp $
+#   $Id: Contract.pm,v 1.20 2008/05/22 16:08:56 erwan_lemonnier Exp $
 #
 
 package Sub::Contract;
@@ -25,9 +25,10 @@ our @EXPORT = qw();
 our @EXPORT_OK = qw( contract
 		     undef_or
 		     defined_and
+		     is_a
 		     );
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 my $pool = Sub::Contract::Pool::get_contract_pool();
 
@@ -77,6 +78,21 @@ sub defined_and {
     };
 }
 
+#---------------------------------------------------------------
+#
+#   is_a - take a package name and return a ref to a sub that
+#          verifies that its argument is an instance of this package
+#
+
+sub is_a {
+    croak "is_a() expects a package name" if (scalar @_ != 1 || !defined $_[0] || ref $_[0] ne '');
+    my $type = shift;
+    return sub {
+	return 0 if (!defined $_[0]);
+	return (ref $_[0] eq $type) ? 1:0;
+    };
+}
+
 ################################################################
 #
 #
@@ -144,7 +160,7 @@ sub reset {
     $self->{pre}         = undef;        # Coderef checking pre conditions
     $self->{post}        = undef;        # Coderef checking post conditions
     $self->{invariant}   = undef;        # Coderef checking an invariant condition
-    $self->{is_memoized} = 0;
+    delete $self->{cache};
     return $self;
 }
 
@@ -300,14 +316,6 @@ __END__
 
 =head1 NAME
 
-WARNING!!!
-
-This is an alfa release!
-Some features are not implemented yet and test coverage is still low!!
-
-WARNING!!!
-
-
 Sub::Contract - Pragmatic contract programming for Perl
 
 =head1 SYNOPSIS
@@ -548,14 +556,10 @@ heavier syntax if you are only seeking to validate input arguements and return v
 Class::Agreement does not provide memoization from within the contract.
 
 
-
-
 TODO: more description
 TODO: how to enable contracts -> enable on each contract, or via the pool
-
 TODO: validation code should not change @_, else weird bugs...
 
-
 =back
 
 =head1 Object API
@@ -820,7 +824,23 @@ Example:
     # or must validate is_word().
     contract('set_name')
         ->in( name => defined_and(\&is_word),
-              nickname => undef_or(\&is_word))
+              nickname => undef_or(\&is_word) )
+        ->enable;
+
+   sub set_name {...}
+
+
+=item C<< is_a($pkg) >>
+
+Returns a subroutine that takes 1 argument and returns
+true if this argument is an instance of C<$pkg> and
+false if not.
+
+Example:
+
+    # argument 'name' must be an instance of String::Name
+    contract('set_name')
+        ->in( name => is_a("String::Name") )
         ->enable;
 
    sub set_name {...}
@@ -883,7 +903,7 @@ See 'Issues with contract programming' under 'Discussion'.
 
 =head1 VERSION
 
-$Id: Contract.pm,v 1.17 2008/05/07 09:08:21 erwan_lemonnier Exp $
+$Id: Contract.pm,v 1.20 2008/05/22 16:08:56 erwan_lemonnier Exp $
 
 =head1 AUTHORS
 
